@@ -55,14 +55,13 @@ static void            bonobo_mdi_class_init     (BonoboMDIClass  *);
 static void            bonobo_mdi_instance_init  (BonoboMDI *);
 static void            bonobo_mdi_finalize       (GObject *);
 static void            child_list_menu_create    (BonoboMDI *, BonoboWindow *);
-static void            child_list_menu_create_all (BonoboMDI *);
-static void            child_list_menu_remove    (BonoboMDI *, BonoboWindow *);
-static void            child_list_menu_remove_all (BonoboMDI *);
-
 
 static void            child_list_activated_cb   (BonoboUIComponent *uic, gpointer user_data, 
 						  const gchar* verbname);
 
+void                   child_list_menu_remove_item(BonoboMDI *, BonoboMDIChild *);
+void                   child_list_menu_add_item   (BonoboMDI *, BonoboMDIChild *);
+
 static void            app_create               (BonoboMDI *, gchar *, const char *);
 static void            app_clone                (BonoboMDI *, BonoboWindow *, const char *);
 static void            app_destroy              (BonoboWindow *, BonoboMDI *);
@@ -87,7 +86,7 @@ static gboolean        toplevel_focus           (BonoboWindow *, GdkEventFocus *
 static void            set_active_view          (BonoboMDI *, GtkWidget *);
 
 /* convenience functions that call child's "virtual" functions */
-static GtkWidget      *child_set_label         (BonoboMDIChild *, GtkWidget *);
+static GtkWidget      *child_set_label         (BonoboMDIChild *, GtkWidget *, GtkWidget *);
 
 static void 	       child_name_changed 	(BonoboMDIChild *mdi_child, 
 						 gchar* old_name, 
@@ -315,8 +314,6 @@ bonobo_mdi_finalize (GObject *object)
 	mdi = BONOBO_MDI (object);
 	g_return_if_fail (mdi->priv != NULL);
 	
-	bonobo_mdi_remove_all (mdi, TRUE);
-
 	if (mdi->priv->child_list_path != NULL)
 		g_free (mdi->priv->child_list_path);
 	
@@ -341,6 +338,21 @@ bonobo_mdi_finalize (GObject *object)
 	gedit_debug (DEBUG_MDI, "END");
 }
 
+void 
+bonobo_mdi_destroy (BonoboMDI *mdi)
+{
+	gedit_debug (DEBUG_MDI, "");
+
+	g_return_if_fail (BONOBO_IS_MDI (mdi));
+	
+	bonobo_mdi_remove_all (mdi, TRUE);
+
+	if (bonobo_mdi_get_active_window (mdi))
+	{
+		gtk_widget_destroy (GTK_WIDGET (bonobo_mdi_get_active_window (mdi)));
+	}
+}
+	
 
 static void 
 bonobo_mdi_instance_init (BonoboMDI *mdi)
@@ -409,10 +421,10 @@ bonobo_mdi_new (const gchar *mdi_name, const gchar *title,
 }
 
 static GtkWidget *
-child_set_label (BonoboMDIChild *child, GtkWidget *label)
+child_set_label (BonoboMDIChild *child, GtkWidget *view, GtkWidget *label)
 {
 	GtkWidget *w;
-	w = BONOBO_MDI_CHILD_GET_CLASS (child)->set_label (child, label, NULL);
+	w = BONOBO_MDI_CHILD_GET_CLASS (child)->set_label (child, view, label, NULL);
 	
 	return w;
 }
@@ -503,7 +515,6 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 {
 	GList *child;
 	BonoboUIComponent *ui_component;
-	gint accell_num = 1;
 
 	if (mdi->priv->child_list_path == NULL)
 		return;
@@ -535,17 +546,9 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 		verb_name = g_strdup_printf ("Child_%p", child->data);
 		xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"%s\""
 				" _label=\"%s\"/>", verb_name, verb_name, escaped_name);
-
-		if (accell_num <= 9)
-			cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
-				" _tip=\"%s\" accel=\"*Alt*%d\"/>", 
-				verb_name, escaped_name, tip, accell_num);
-		else
-			cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
+		cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
 				" _tip=\"%s\"/>", verb_name, escaped_name, tip);
-	
-		++ accell_num;
-		
+
 		g_free (tip);
 		g_free (child_name);
 		g_free (safe_name);
@@ -565,84 +568,107 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 	bonobo_ui_component_thaw (ui_component, NULL);
 }
 
-static void 
-child_list_menu_remove (BonoboMDI *mdi, BonoboWindow *win)
-{
-	GList *child;
-	BonoboUIComponent *ui_component;
 
-	if (mdi->priv->child_list_path == NULL)
+void 
+child_list_menu_remove_item (BonoboMDI *mdi, BonoboMDIChild *child)
+{
+	GList *win_node;
+	gchar *path, *cmd, *verb_name;
+	
+	gedit_debug (DEBUG_MDI, "");
+
+	if(mdi->priv->child_list_path == NULL)
 		return;
 	
-	ui_component = BONOBO_UI_COMPONENT (
-			g_object_get_data (G_OBJECT (win), UI_COMPONENT_KEY));
-			
-	child = mdi->priv->children;
-	
-	bonobo_ui_component_freeze (ui_component, NULL);
-	
-	while (child) 
+	win_node = mdi->priv->windows;
+
+	verb_name = g_strdup_printf ("Child_%p", child);
+	path = g_strdup_printf ("%s%s", mdi->priv->child_list_path, verb_name);
+	cmd = g_strdup_printf ("/commands/%s", verb_name);
+
+	while (win_node) 
 	{
-		gchar *verb_name = NULL;
-		gchar *path;
-		gchar *cmd;
+		BonoboUIComponent *ui_component;
+		
+		ui_component = BONOBO_UI_COMPONENT (
+			g_object_get_data (G_OBJECT (win_node->data), UI_COMPONENT_KEY));
 		
-		verb_name = g_strdup_printf ("Child_%p", child->data);
-		path = g_strdup_printf ("%s%s", mdi->priv->child_list_path, verb_name);
-		cmd = g_strdup_printf ("/commands/%s", verb_name);
-	
 		bonobo_ui_component_remove_verb (ui_component, verb_name);
 		bonobo_ui_component_rm (ui_component, path, NULL);
 		bonobo_ui_component_rm (ui_component, cmd, NULL);
-
-		g_free (path); 
-		g_free (cmd);
-		g_free (verb_name);
-
-		child = g_list_next (child);
+		win_node = g_list_next (win_node);
 	}
 
-	bonobo_ui_component_thaw (ui_component, NULL);
+	g_free (path);
+	g_free (cmd);
+	g_free (verb_name);
+
+	gedit_debug (DEBUG_MDI, "END");
 }
 
-static void 
-child_list_menu_create_all (BonoboMDI *mdi)
+
+void 
+child_list_menu_add_item (BonoboMDI *mdi, BonoboMDIChild *child)
 {
 	GList *win_node;
-
-	if (mdi->priv->child_list_path == NULL)
+	gchar *child_name, *escaped_name, *safe_name;
+	gchar* xml, *cmd, *verb_name, *tip;
+	int accel_num;
+	
+	if(mdi->priv->child_list_path == NULL)
 		return;
 
+	accel_num = g_list_length (mdi->priv->children);
+	
 	win_node = mdi->priv->windows;
+
+	child_name = bonobo_mdi_child_get_name (child);
+
+	safe_name = g_markup_escape_text (child_name, strlen (child_name));
+	g_return_if_fail (safe_name != NULL);
+
+	escaped_name = escape_underscores (safe_name);
+	g_return_if_fail (escaped_name != NULL);
+
+	verb_name = g_strdup_printf ("Child_%p", child);
+	
+	tip = g_strdup_printf (_("Activate %s"), safe_name);
+	xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"%s\""
+				" _label=\"%s\"/>", verb_name, verb_name, escaped_name);
+
+	if (accel_num > 9)
+		cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
+				" _tip=\"%s\"/>", verb_name, escaped_name, tip);
+	else
+		cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
+			" _tip=\"%s\" accel=\"*Alt*%d\"/>", verb_name,
+			escaped_name, tip, accel_num);
+
+	
+	while (win_node) 
+	{
+		BonoboUIComponent *ui_component;
 		
-	while (win_node) 
-	{
-		child_list_menu_create (mdi, BONOBO_WINDOW (win_node->data));
+		ui_component = BONOBO_UI_COMPONENT (
+			g_object_get_data (G_OBJECT (win_node->data), UI_COMPONENT_KEY));
+		
+
+		bonobo_ui_component_set_translate (ui_component, mdi->priv->child_list_path, xml, NULL);
+		bonobo_ui_component_set_translate (ui_component, "/commands/", cmd, NULL);
+		bonobo_ui_component_add_verb (ui_component, verb_name, child_list_activated_cb, child); 
 
 		win_node = g_list_next (win_node);
 	}
+	
+	g_free (tip);
+	g_free (escaped_name);
+	g_free (child_name);
+	g_free (xml);
+	g_free (cmd);
+	g_free (verb_name);
+	g_free (safe_name);
 }
 
-static void 
-child_list_menu_remove_all (BonoboMDI *mdi)
-{
-	GList *win_node;
-
-	if (mdi->priv->child_list_path == NULL)
-		return;
-
-	win_node = mdi->priv->windows;
-
-	while (win_node) 
-	{
-		child_list_menu_remove (mdi, BONOBO_WINDOW (win_node->data));
-
-		win_node = g_list_next (win_node);
-	}
-}
-
-
-
 static gboolean 
 book_motion (GtkWidget *widget, GdkEventMotion *e, gpointer data)
 {
@@ -862,7 +888,7 @@ book_add_view (GtkNotebook *book, GtkWidget *view)
 
 	child = bonobo_mdi_get_child_from_view (view);
 
-	title = child_set_label (child, NULL);
+	title = child_set_label (child, view, NULL);
 
 	gtk_notebook_append_page (book, view, title);
 
@@ -917,10 +943,10 @@ toplevel_focus (BonoboWindow *win, GdkEventFocus *event, BonoboMDI *mdi)
 
 	/* updates active_view and active_child when a new toplevel receives focus */
 	g_return_val_if_fail (BONOBO_IS_WINDOW (win), FALSE);
-
+	
 	if (mdi->priv->active_window == win)
 		return FALSE;
-	
+
 	mdi->priv->active_window = win;
 	
 	contents = get_book_from_window (win);
@@ -1624,14 +1650,12 @@ bonobo_mdi_add_child (BonoboMDI *mdi, BonoboMDIChild *child)
 
 	bonobo_mdi_child_set_parent (child, G_OBJECT (mdi));
 
-	child_list_menu_remove_all (mdi);
-	
 	mdi->priv->children = g_list_append (mdi->priv->children, child);
 
 	g_signal_connect (G_OBJECT (child), "name_changed",
 			  G_CALLBACK (child_name_changed), mdi);
 
-	child_list_menu_create_all (mdi);
+	child_list_menu_add_item (mdi, child);
 
 	g_object_set_data (G_OBJECT (child), BONOBO_MDI_KEY, mdi);
 
@@ -1691,11 +1715,9 @@ bonobo_mdi_remove_child (BonoboMDI *mdi, BonoboMDIChild *child, gint force)
 		bonobo_mdi_remove_view (mdi, GTK_WIDGET (view), TRUE);
 	}
 
-	child_list_menu_remove_all (mdi);
-	
 	mdi->priv->children = g_list_remove (mdi->priv->children, child);
 
-	child_list_menu_create_all (mdi);
+	child_list_menu_remove_item (mdi, child);
 
 	if (child == mdi->priv->active_child)
 		mdi->priv->active_child = NULL;
@@ -1833,7 +1855,7 @@ bonobo_mdi_update_child (BonoboMDI *mdi, BonoboMDIChild *child)
 	{
 		view = GTK_WIDGET (view_node->data);
 
-		title = child_set_label (child, NULL);
+		title = child_set_label (child, view, NULL);
 		
 		gtk_notebook_set_tab_label (GTK_NOTEBOOK (view->parent), view, title);
 		
