@@ -191,6 +191,7 @@ gedit_file_open (GeditMDIChild *active_child)
 static gboolean
 gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 {
+	EggRecentItem *item;
 	GError *error = NULL;
 	gchar *uri;
 	
@@ -247,7 +248,10 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 	}
 	
 	recent = gedit_recent_get_model ();
-	egg_recent_model_add (recent, uri);
+	item = egg_recent_item_new_from_uri (uri);
+	egg_recent_item_add_group (item, "gedit");
+	egg_recent_model_add_full (recent, item);
+	egg_recent_item_unref (item);
 
 	g_free (uri);
 
@@ -257,7 +261,7 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 }
 
 gboolean 
-gedit_file_save (GeditMDIChild* child)
+gedit_file_save (GeditMDIChild* child, gboolean force)
 {
 	gint ret;
 	GeditDocument* doc = NULL;
@@ -278,7 +282,7 @@ gedit_file_save (GeditMDIChild* child)
 		return gedit_file_save_as (child);
 	}
 
-	if (!gedit_document_get_modified (doc))	
+	if (!force && !gedit_document_get_modified (doc))	
 	{
 		gchar *raw_uri;
 		gboolean deleted = FALSE;
@@ -339,6 +343,7 @@ gedit_file_save (GeditMDIChild* child)
 	else
 	{
 		EggRecentModel *recent;
+		EggRecentItem *item;
 		gchar *raw_uri;
 		
 		gedit_debug (DEBUG_FILE, "OK");
@@ -351,7 +356,10 @@ gedit_file_save (GeditMDIChild* child)
 		g_return_val_if_fail (raw_uri != NULL, TRUE);
 		
 		recent = gedit_recent_get_model ();
-		egg_recent_model_add (recent, raw_uri);
+		item = egg_recent_item_new_from_uri (raw_uri);
+		egg_recent_item_add_group (item, "gedit");
+		egg_recent_model_add_full (recent, item);
+		egg_recent_item_unref (item);
 
 		g_free (raw_uri);
 		
@@ -497,9 +505,13 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	else
 	{
 		EggRecentModel *recent;
+		EggRecentItem *item;
 
 		recent = gedit_recent_get_model ();
-		egg_recent_model_add (recent, uri);
+		item = egg_recent_item_new_from_uri (uri);
+		egg_recent_item_add_group (item, "gedit");
+		egg_recent_model_add_full (recent, item);
+		egg_recent_item_unref (item);
 
 		g_free (uri);
 
@@ -528,20 +540,15 @@ gedit_file_close_all (void)
 void
 gedit_file_exit (void)
 {
-	BonoboWindow *active_window;
-	
 	gedit_debug (DEBUG_FILE, "");
 	
 	if (!gedit_file_close_all ())
 		return;
 
-	active_window = gedit_get_active_window ();
-
-	if (active_window != NULL)
-		gedit_prefs_manager_save_window_size_and_state (active_window);
-
 	gedit_debug (DEBUG_FILE, "All files closed.");
 	
+	bonobo_mdi_destroy (BONOBO_MDI (gedit_mdi));
+
 	gedit_plugins_engine_save_settings ();
 	
 	gedit_prefs_manager_shutdown ();
@@ -577,10 +584,7 @@ gedit_file_save_all (void)
 		child = GEDIT_MDI_CHILD (g_list_nth_data (
 				bonobo_mdi_get_children (BONOBO_MDI (gedit_mdi)), i));
 
-		if (gedit_document_get_modified (child->document))
-		{
-			gedit_file_save (child);	
-		}
+		gedit_file_save (child, FALSE);	
 	}
 
 	if (view !=  bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)))
